# -*- coding: utf-8 -*-
from abc import ABC, abstractmethod
from typing import Optional

from pip_services4_components.context import IContext
from pip_services4_data.query import FilterParams, PagingParams, DataPage

from ..data import GroupMembershipV1


class IGroupMembershipsPersistence(ABC):

    @abstractmethod
    def get_page_by_filter(self, context: Optional[IContext], filter_params: FilterParams,
                            paging: PagingParams) -> DataPage:
        pass

    @abstractmethod
    def get_one_by_id(self, context: Optional[IContext], id: str) -> Optional[GroupMembershipV1]:
        pass

    @abstractmethod
    def create(self, context: Optional[IContext], item: GroupMembershipV1) -> GroupMembershipV1:
        pass

    @abstractmethod
    def update(self, context: Optional[IContext], item: GroupMembershipV1) -> GroupMembershipV1:
        pass

    @abstractmethod
    def delete_by_id(self, context: Optional[IContext], id: str) -> Optional[GroupMembershipV1]:
        pass

    @abstractmethod
    def delete_by_filter(self, context: Optional[IContext], filter_params: FilterParams) -> None:
        pass
